# echo "Picking dependency packages from NAS /custom_plugin/data_quality";
# pip install pandas-profiling==3.6.6;
# pip install bs4==0.0.1;
# pip install refractio[all]==2.0.5.9;
# echo "Done with dependency installtion!";
pip install feast